import rawMenus from '@/config/menu'
import pluralize from 'pluralize'
import { deepCopy } from './deepcopy';

function capitalize(s) {
    return s.replace(/\b(\w)(\w*)/g, ($0, $1, $2) => {
        return $1.toUpperCase() + $2.toLowerCase();
    })
}

/**
 * 获取所有的菜单数据
 * @returns
 */
export function getMenus() {
    let cloneMenus = deepCopy(rawMenus)
    return cloneMenus
}

/**
 *  序列化对象
 * @param {Object} menu 
 * @returns 
 */
export function nameize(menu) {
    let componentName = menu.name || '';
    if (componentName === '') {
        componentName = menu.route.substr(1).split('/').map(v => {
            return v[0] === ":" ? "" : capitalize(pluralize.singular(v));
        }).join('')
    }
    return componentName
}

/**
 * 查找指定路由的导航路径
 * @param {Array} values 
 * @param {String} target 
 */
export function findNavigation(values, target) {
    let paths = [];
    for (let i in values) {
        let row = values[i];
        if (row.route && row.route === target) {
            paths.push({
                navigation: row.navigation || false,
                label: row.label,
                route: row.route
            })
            return paths;
        }
        if (row.children && Array.isArray(row.children)) {
            let vs = findNavigation(row.children, target);
            if (Array.isArray(vs)) {
                paths.push({
                    navigation: row.navigation || false,
                    label: row.label,
                    route: row.route
                });
                for (let j in vs) {
                    paths.push(vs[j])
                }
                return paths;
            }
        }
    }
    return false;
}

/**
 * 通过菜单配置构建菜单菜单路由组件
 * @param {Array} rows 
 * @param {Object} components 
 * @returns 
 */
export function buildRoutes(rows, components) {
    let routes = [];
    for (let menu of rows) {
        let route = { name: '', path: menu.route || '/', meta: {}, component: null, props: {} }
        route.meta.title = menu.label || '';
        route.props.title = route.meta.title
        route.name = nameize(menu)
        route.meta.hidden = menu.hidden ? true : false
        route.meta.cache = menu.cache ? true : false
        if (menu.permissions) {
            route.meta.permissions = menu.permissions
        }
        let viewPath = menu.view;
        if (!viewPath) {
            let tokens = menu.route.substr(1).split('/').map((v) => {
                return v[0] === ":" ? "" : v;
            }).filter((v) => {
                return v !== "";
            })
            let totalLength = tokens.length;
            if (totalLength > 0) {
                if (totalLength == 1 || pluralize.isPlural(tokens[totalLength - 1])) {
                    if (pluralize.isPlural(tokens[totalLength - 1])) {
                        tokens[totalLength - 1] = pluralize.singular(tokens[totalLength - 1])
                    }
                    tokens.push('index');
                    totalLength++;
                }
                tokens[totalLength - 1] = capitalize(tokens[totalLength - 1]);
            }
            viewPath = '../views/' + tokens.join("/") + ".vue"
        }
        if (components[viewPath]) {
            route.component = components[viewPath]
        }
        if (Array.isArray(menu.children)) {
            for (let child of buildRoutes(menu.children, components)) {
                if (!child.meta) {
                    child.meta = {};
                }
                child.meta.children = true;
                child.meta.route = {
                    parent: menu.route
                }
                routes.push(child);
            }
        }
        routes.push(route);
    }
    return routes;
}

/**
 * 判断该路由是否有权限访问
 * @param {Object} row 
 * @param {Array} permissions 
 * @returns 
 */
function isAllowed(row, permissions) {
    if (row.access && row.access == 'allow') {
        return true
    }
    let name = nameize(row)
    if (permissions.indexOf(name) > -1) {
        return true
    }
    //只要有页面下面一个权限，那么就要显示该页面
    if (Array.isArray(row.permissions)) {
        for (let per of row.permissions) {
            if (permissions.indexOf(name + '.' + per)) {
                return true
            }
        }
    }
    //只要有一个子项目有权限，那么他就有权限
    if (Array.isArray(row.children)) {
        for (let child of row.children) {
            if (isAllowed(child, permissions)) {
                return true;
            }
        }
    }
    return false
}

function filterMenusInner(items, permissions) {
    let data = [];
    for (let i in items) {
        let row = Object.assign({}, items[i]);
        if (!row.hidden) {
            if (isAllowed(row, permissions)) {
                if (Array.isArray(row.children)) {
                    row.children = filterMenusInner(row.children, permissions)
                }
                data.push(row)
            }
        }
    }
    return data
}

/**
 * 过滤有权限的菜单
 * @param {Array} permissions 
 * @returns 
 */
export function filterMenus(permissions) {
    return filterMenusInner(getMenus(), permissions)
}